VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CTimeZoneInfo"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' ***************************************************************************
' * Description:
' *   This object retrieves system or specific time zone information from the
' *   registry and/or api call.
' *
' *
' * Methods (Private):
' *   DateFromSYSTEMTIME - Returns either a Date or Null using values in SYSTEMTIME structure
' *
' *
' * Methods (Public):
' *   DateInDaylightRange - Returns True if dtDate is between DaylightStartDate and StandardStartDate
' *   Refresh - Fills time zone information from registry and/or api call
' *
' *
' * Properties:
' *   Bias (read-only) - Specifies the minutes between Coordinated Universal Time (UTC) and local time.
' *                      local time = UTC - bias + (daylight bias or standard bias)
' *   DaylightBias (read-only) - Specifies minutes to add to Bias to get local time during Daylight Savings Time.
' *   DaylightShortName (read-only) - Short Name for Daylight Savings Time; ex: "Mountain Daylight Time"
' *   DaylightStartDate (read-only) - Returns Date when Daylight Savings Time starts or Null if timezone doesn't support Daylight Time
' *   DisplayName (read-only) - Name of time zone; ex: "(GMT-07:00) Mountain Time (US & Canada)"
' *   StandardBias (read-only) - Specified minutes to add to Bias to get local time during Standard Time (usually 0).
' *   StandardShortName (read-only) - Short Name for Standard Time; ex: "Mountain Standard Time"
' *   StandardStartDate (read-only) - Returns Date when Standard Time starts or Null
' *   SupportDaylight (read-only) - Indicates if specified time zone supports Daylight Savings Time
' *   TimeZoneID (read/write) - Sets/returns the TimeZoneConstants enum value; Changing this value refreshes the object.
' *
' *
' * Associations:
' *  RegKey - Used to read values from the registry.
' *  CSystem - Exposes Platform property to determine if NT or 95/Other is running
' *  modDateFunctions - Common date functions
' *
' *
' *
' * Events:
' *
' *
' * Revisions:
' *   07/27/1998 RMT Created
' *
' ****************************************************************************

Option Explicit

' Mod INTERFACES


' Mod EVENTS


' Mod CONSTANTS
Private Const MODULE As String = "CTimeZoneInfo"



' Mod TYPES
Private Type SYSTEMTIME
        wYear As Integer
        wMonth As Integer
        wDayOfWeek As Integer
        wDay As Integer
        wHour As Integer
        wMinute As Integer
        wSecond As Integer
        wMilliseconds As Integer
End Type

Private Type TIME_ZONE_INFORMATION_REG
        Bias As Long
        StandardBias As Long
        DaylightBias As Long
        StandardDate As SYSTEMTIME
        DaylightDate As SYSTEMTIME
End Type

Private Type TIME_ZONE_INFORMATION
        Bias As Long
        StandardName As String * 64
        StandardDate As SYSTEMTIME
        StandardBias As Long
        DaylightName As String * 64
        DaylightDate As SYSTEMTIME
        DaylightBias As Long
End Type



' Mod DECLARES
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (lpvDest As Any, lpvSource As Any, ByVal cbCopy As Long)
Private Declare Function GetTimeZoneInformation Lib "kernel32" (lpTimeZoneInformation As TIME_ZONE_INFORMATION) As Long


' Mod VARIABLES
Private mlngID As TimeZoneConstants
Private mstrDisplayName As String
Private mstrStandardShortName As String
Private mstrDaylightShortName As String
Private mstrStandardAbbrev As String
Private mstrDaylightAbbrev As String
Private mtzrTimeZoneInfoRegistryStruct As TIME_ZONE_INFORMATION_REG
Private mbolSupportDaylight As Boolean

'MOD ENUMS
Public Enum TimeZoneConstants
    tzcTimeZoneUnknown = -2
    tzcTimeZoneSystem = -1
    tzcTimeZoneStandard = 0
    tzcTimeZoneDaylight = 1
    tzcTimeZoneGMT = 2
    tzcTimeZonePacific = 4
    tzcTimeZoneMountain = 6
    tzcTimeZoneCentral = 8
    tzcTimeZoneEastern = 10
    tzcTimeZoneAtlantic = 12
    tzcTimeZoneUSMountain = 14
End Enum
'TimeZoneConstants are multiples of two;
'even numbers denote Standard Time while odd numbers denote Daylight Time
'
'To get Daylight Savings Time add tzcTimeZoneDaylight to tzcTimeZoneGMT or above
'  ex: MDT = tzcTimeZoneMountain + tzcTimeZoneDaylight (value = 7)
'      MST = tzcTimeZoneMountain or tzcTimeZoneMountain + tzcTimeZoneStandard (value = 6)

Public Property Get TimeZoneID() As TimeZoneConstants
   
    TimeZoneID = mlngID

End Property
Public Property Let TimeZoneID(ByVal aValue As TimeZoneConstants)
On Error GoTo ErrRtn


    If aValue <> mlngID Then
        'value changed, refresh the object
        mlngID = aValue
        Refresh
    End If


Xit:
    GoSub ClearObjects
    Exit Property

ErrRtn:
    GoSub ClearObjects
    Err.Raise vbObjectError, MODULE & ".TimeZoneID", Err.Description
    Resume Xit
    Resume

ClearObjects:
    Return


End Property

Public Property Get StandardStartDate(ByVal iYear As Integer) As Variant
    
    'returns either a Date or Null
    StandardStartDate = DateFromSYSTEMTIME(mtzrTimeZoneInfoRegistryStruct.StandardDate, iYear)

End Property
Public Property Get DaylightStartDate(ByVal iYear As Integer) As Variant

    'returns either a Date or Null
    DaylightStartDate = DateFromSYSTEMTIME(mtzrTimeZoneInfoRegistryStruct.DaylightDate, iYear)

End Property
Public Property Get DaylightBias() As Long

    DaylightBias = mtzrTimeZoneInfoRegistryStruct.DaylightBias

End Property
Public Property Get StandardBias() As Long

    StandardBias = mtzrTimeZoneInfoRegistryStruct.StandardBias

End Property
Public Property Get Bias() As Long

    Bias = mtzrTimeZoneInfoRegistryStruct.Bias

End Property
Public Property Get DaylightShortName() As String

    DaylightShortName = mstrDaylightShortName

End Property
Public Property Get StandardShortName() As String

    StandardShortName = mstrStandardShortName

End Property

Public Property Get DisplayName() As String

    DisplayName = mstrDisplayName

End Property


Public Sub Refresh()
On Error GoTo ErrRtn

'Fills time zone information from registry and/or api call

    Dim strTimeZoneKey As String
    Dim vRet As Variant
    Dim bytArray() As Byte
    Dim tziTimeZoneInfoStruct As TIME_ZONE_INFORMATION
    Dim oSystem As New CSystem
    Dim objRegKey As RegKey
    Dim sKey As String
   

    If mlngID = tzcTimeZoneSystem Then
        'get current system time zone settings
        
        vRet = GetTimeZoneInformation(tziTimeZoneInfoStruct)
        
        With tziTimeZoneInfoStruct
            mstrStandardShortName = StripNull(StrConv(.StandardName, vbFromUnicode))
            mstrDaylightShortName = StripNull(StrConv(.DaylightName, vbFromUnicode))
            mtzrTimeZoneInfoRegistryStruct.Bias = .Bias
            mtzrTimeZoneInfoRegistryStruct.DaylightBias = .DaylightBias
            mtzrTimeZoneInfoRegistryStruct.DaylightDate = .DaylightDate
            mtzrTimeZoneInfoRegistryStruct.StandardBias = .StandardBias
            mtzrTimeZoneInfoRegistryStruct.StandardDate = .StandardDate
        End With
        

        mlngID = GetIDFromShortName(mstrStandardShortName)
        strTimeZoneKey = GetTimeZoneRegKeyFromID(mlngID)
        If Len(strTimeZoneKey) > 0 Then
        
            If oSystem.Platform = emsOSPlatformWindowsNT Then
                sKey = TZ_REG_KEY_NT & strTimeZoneKey
            Else
                sKey = TZ_REG_KEY_95 & strTimeZoneKey
            End If
        
            Set objRegKey = RegKeyFromString("\HKEY_LOCAL_MACHINE\" & sKey)
                        
            'DisplayName is not returned in TIME_ZONE_INFORMATION structure, read from registry
            mstrDisplayName = objRegKey.Values.Item("Display").Value
        End If
    
                
    Else
        'get specified time zone from registry
        
        strTimeZoneKey = GetTimeZoneRegKeyFromID(mlngID)
        If Len(strTimeZoneKey) > 0 Then
        
            If oSystem.Platform = emsOSPlatformWindowsNT Then
                sKey = TZ_REG_KEY_NT & strTimeZoneKey
            Else
                sKey = TZ_REG_KEY_95 & strTimeZoneKey
            End If
            
            
            Set objRegKey = RegKeyFromString("\HKEY_LOCAL_MACHINE\" & sKey)
            mstrDisplayName = objRegKey.Values.Item("Display").Value
            mstrDaylightShortName = objRegKey.Values.Item("Dlt").Value
            mstrStandardShortName = objRegKey.Values.Item("Std").Value

            vRet = objRegKey.Values.Item("TZI").Value
            
            If IsEmpty(vRet) Then
                Err.Raise vbObjectError, , "Registry key or value not valid."
            End If
                
             
            bytArray = vRet 'assigns byte array in variant to dynamic byte array
            
            'copy bytes into appropriate structure locations
            CopyMemory mtzrTimeZoneInfoRegistryStruct, bytArray(0), Len(mtzrTimeZoneInfoRegistryStruct)
            
        End If
            
    End If

    'if the wMonth entry is 0 then time zone doesn't support daylight savings
    'and therefore will not have values for StandardDate and DaylightDate
    If mtzrTimeZoneInfoRegistryStruct.StandardDate.wMonth = 0 Then
        mbolSupportDaylight = False
    Else
        mbolSupportDaylight = True
    End If
    

Xit:
    GoSub ClearObjects
    Exit Sub

ErrRtn:
    GoSub ClearObjects
    Err.Raise vbObjectError, MODULE & ".Refresh", Err.Description
    Resume Xit
    Resume

ClearObjects:
    Erase bytArray
    Return

End Sub

Public Property Get SupportDaylight() As Boolean
    SupportDaylight = mbolSupportDaylight
End Property

Public Function DateInDaylightRange(ByVal dtDate As Date) As Boolean
'Returns True if dtDate is between DaylightStartDate and StandardStartDate
Dim iYear As Integer
On Error GoTo ErrRtn

    DateInDaylightRange = False

    iYear = Year(dtDate)

    If SupportDaylight = True Then
        If dtDate >= DaylightStartDate(iYear) And dtDate < StandardStartDate(iYear) Then
            DateInDaylightRange = True
        End If
    End If

Xit:
    GoSub ClearObjects
    Exit Function

ErrRtn:
    GoSub ClearObjects
    Err.Raise vbObjectError, MODULE & ".DateInDaylightRange", Err.Description
    Resume Xit
    Resume

ClearObjects:
    Return

    
End Function

Private Function DateFromSYSTEMTIME(aSYSTEMTIME As SYSTEMTIME, Optional iRefYear As Integer) As Variant
'Returns either a Date or Null using values in SYSTEMTIME structure
On Error GoTo ErrRtn

With aSYSTEMTIME
    
    If .wDay = 0 And .wDayOfWeek = 0 And .wHour = 0 And .wMilliseconds = 0 And _
        .wMinute = 0 And .wMonth = 0 And .wSecond = 0 And .wYear = 0 Then
        
        'all structure fields contain 0 indicating invalid date
        DateFromSYSTEMTIME = Null
        
    Else
    
        If .wYear = 0 Then
            'day-in-month format; uses wMonth, wDayOfWeek, wDay to identify a specific occurance of
            'a weekday in that month.  The year is specified by iRefYear or the current year.
            
            'if no year passed in, use current year
            If iRefYear = 0 Then iRefYear = Year(Date)
            
            DateFromSYSTEMTIME = WeekdayOccurance(iRefYear, .wMonth, (.wDayOfWeek + 1), .wDay) + TimeSerial(.wHour, .wMinute, .wSecond)
        Else
            'absolute format; includes year
            DateFromSYSTEMTIME = DateSerial(.wYear, .wMonth, .wDay) + TimeSerial(.wHour, .wMinute, .wSecond)
        End If
        
    End If

End With


Xit:
    GoSub ClearObjects
    Exit Function

ErrRtn:
    GoSub ClearObjects
    Err.Raise vbObjectError, MODULE & ".DateFromSYSTEMTIME", Err.Description
    Resume Xit
    Resume

ClearObjects:
    Return


End Function


Private Sub Class_Initialize()
    TimeZoneID = tzcTimeZoneSystem
End Sub

Private Function StripNull(sValue As String) As String
Dim lPos As Long

    lPos = InStr(sValue, Chr(0))
    If lPos > 0 Then
        StripNull = Left$(sValue, lPos - 1)
    End If
End Function


